package com.casic.missiles.mapper.finance;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.casic.missiles.dto.cockpit.ManageTrendResponse;
import com.casic.missiles.dto.finance.SettlementInfoDetailResponse;
import com.casic.missiles.dto.finance.SettlementInfoListRequest;
import com.casic.missiles.dto.finance.SettlementInfoListResponse;
import com.casic.missiles.model.finance.FinanceBusinessSettlementInfo;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 业务结算表-未在合同里的委托书收款信息 Mapper 接口
 * </p>
 *
 * @author wangpeng
 * @since 2023-03-31
 */
public interface FinanceBusinessSettlementInfoMapper extends BaseMapper<FinanceBusinessSettlementInfo> {

    Page<SettlementInfoListResponse> selectListPage(Page<SettlementInfoListResponse> page, @Param("request") SettlementInfoListRequest request);

    List<SettlementInfoListResponse> selectList(@Param("request") SettlementInfoListRequest request);

    SettlementInfoDetailResponse selectDetailById(@Param("id") Long id);

    List<SettlementInfoListResponse> selectListByIds(@Param("ids")List<Long> ids);

    List<SettlementInfoListResponse> selectListByDimension(@Param("dimension") String dimension);

    List<ManageTrendResponse> selectPreMonthAmountByYear();
}
